/****************************************************************************************** 
 *	Chili DirectX Framework Version 11.12.17											  *	
 *	Game.h																				  *
 *	Copyright 2011 PlanetChili.net														  *
 *																						  *
 *	This file is part of The Chili DirectX Framework.									  *
 *																						  *
 *	The Chili DirectX Framework is free software: you can redistribute it and/or modify	  *
 *	it under the terms of the GNU General Public License as published by				  *
 *	the Free Software Foundation, either version 3 of the License, or					  *
 *	(at your option) any later version.													  *
 *																						  *
 *	The Chili DirectX Framework is distributed in the hope that it will be useful,		  *
 *	but WITHOUT ANY WARRANTY; without even the implied warranty of						  *
 *	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the						  *
 *	GNU General Public License for more details.										  *
 *																						  *
 *	You should have received a copy of the GNU General Public License					  *
 *	along with The Chili DirectX Framework.  If not, see <http://www.gnu.org/licenses/>.  *
 ******************************************************************************************/
#pragma once

#include "D3DGraphics.h"
#include "Keyboard.h"
#define BULLETNUMBER 200 
#define STARNUMBER 100
#define BADSHIPNUMBER 20
#define DEBRISNUMBER 29
#define BADSHIPWIDTH 27

class Game
{
public:
	Game( HWND hWnd,const KeyboardServer& kServer );
	void Go();
private:
	void ComposeFrame();
	/********************************/
	/*  User Functions              */
	void DrawFilledCircle( int cx, int cy, int rad, int r, int g, int b );

	//User-Controlled Ship
	void DrawSpaceShip( int x, int y );
	

	//Functions pertaining to BadShip

	void DrawBadShip( int x, int y );
	void SpawnBadShips();
	void MoveBadShips();
	void KillBadShips();
	void ExplosionLeft( int x, int y );
	void ExplosionRight(int x, int y );
	void DestroyLeftWing();
	void DestroyRightWing();
	void DrawDamage();

	//Functions pertaining to Stars

	void PutStars();
	void DrawStars();
	void DrawStar( int x, int y );
	void ControlStarSpeed();

	//Functions pertaining to shooting

	void DrawBullet( int x, int y );
	void FireGun1();
	void FireGun2();
	void FireGun3();
	void FireGun4();

	//Functions pertaining to logic-checks

	void BoundaryCheck();
	void CheckUserInput();
	void CheckForHit();

	//Functions pertaining to visual effects

	void AnimateEngines();
	void DrawEngineFlareStg1( int x, int y);
	void DrawEngineFlareStg2( int x, int y);
	void DrawThrusterRStg1( int x, int y );
	void DrawThrusterRStg2( int x, int y );
	void DrawThrusterRStg3( int x, int y );
	void DrawThrusterRStg4( int x, int y );
	void DrawThrusterRStg5( int x, int y );
	void DrawThrusterLStg1( int x, int y );
	void DrawThrusterLStg2( int x, int y );
	void DrawThrusterLStg3( int x, int y );
	void DrawThrusterLStg4( int x, int y );
	void DrawThrusterLStg5( int x, int y );
	void AnimateThrustersRight();
	void AnimateThrustersLeft();
	void BulletHitAnim( int x, int y);
	void DrawHit1( int x, int y );
	void DrawHit2( int x, int y );
	void DrawHit3( int x, int y );
	void MakeDebris();
	


	// TODO: User functions go here

	/********************************/
	private:
	D3DGraphics gfx;
	KeyboardClient kbd;
	/********************************/
	/*  User Variables              */
	
	int x;
	int y;
	int rad;

	//User-controlled ship variables

	int ShipX;
	int ShipY;

	//BadShip variables

	int BadShipX[BADSHIPNUMBER];
	int BadShipY[BADSHIPNUMBER];
	int BadShipSpeedX[BADSHIPNUMBER];
	int BadShipSpeedY[BADSHIPNUMBER];
	bool BadShipJustSpawned[BADSHIPNUMBER];
	int BadShipMoveTimer[BADSHIPNUMBER];
	int BadShipHealth[BADSHIPNUMBER];
	bool BadShipWingL[BADSHIPNUMBER];
	bool BadShipWingR[BADSHIPNUMBER];
	bool BadShipExists[BADSHIPNUMBER];
	int BadShipPixelX[BADSHIPWIDTH][BADSHIPNUMBER];
	bool BadShipPixelShot[BADSHIPWIDTH][BADSHIPNUMBER];
	bool BadPixelShotLayer1[BADSHIPWIDTH][BADSHIPNUMBER];
	bool BadPixelShotLayer2[BADSHIPWIDTH][BADSHIPNUMBER];
	bool BadPixelShotLayer3[BADSHIPWIDTH][BADSHIPNUMBER];
	bool BadPixelShotLayer4[BADSHIPWIDTH][BADSHIPNUMBER];
	int BB;
	int SpawnTimer;

	//Star variables

	int StarX[STARNUMBER];
	int StarY[STARNUMBER];
	int StarSpeed[STARNUMBER];
	bool StarExists[STARNUMBER];
	int StarN;
	int StarCounter;
	bool NeedStar;
	bool StarSpeedSet[STARNUMBER];

	//Bullet variables

	int BulletX[BULLETNUMBER];
	int BulletY[BULLETNUMBER];
	int BulletHitX[BULLETNUMBER];
	int BulletHitY[BULLETNUMBER];
	bool bulletExists[BULLETNUMBER];
	bool BulletHit[BULLETNUMBER];
	bool BulletExists[BULLETNUMBER];
	bool BulletHitStg1[BULLETNUMBER];
	bool BulletHitStg2[BULLETNUMBER];
	bool BulletHitStg3[BULLETNUMBER];
	int b;
	bool DebrisExists[BULLETNUMBER];
	float DebrisX[BULLETNUMBER];
	float DebrisY[BULLETNUMBER];
	float DebrisSpeedX[BULLETNUMBER];
	int DebrisSpeedY[BULLETNUMBER];
	int bulletX;
	int bulletY;
	
	//Gun variables

	bool Gun1Shot;
	bool Gun2Shot;
	bool Gun3Shot;
	bool Gun4Shot;
	bool GunDelay;

	//Thruster/Engine variables

	bool Stg1Done;
	int EngineTimer;
	bool ThrusterRStg1Done;
	bool ThrusterRStg2Done;
	bool ThrusterRStg3Done;
	bool ThrusterRStg4Done;
	bool ThrusterRStg5Done;
	bool ThrusterLStg1Done;
	bool ThrusterLStg2Done;
	bool ThrusterLStg3Done;
	bool ThrusterLStg4Done;
	bool ThrusterLStg5Done;

	//Explosion variables
	
	bool ExplosionBegun;
	bool ExplosionRStg1Done;
	bool ExplosionRStg2Done;
	bool ExplosionRStg3Done;
	bool ExplosionRStg4Done;
	bool ExplosionRStg5Done;
	bool ExplosionRStg6Done;
	int ExplosionTimerR[BULLETNUMBER];
	int ExplosionTimerL[BULLETNUMBER];
	bool ExplosionRGo[BADSHIPNUMBER];
	bool ExplosionLGo[BADSHIPNUMBER];
	bool InitiateExplosionR[BADSHIPNUMBER];
	/********************************/
};